'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isEmpty = exports.isNotEmpty = exports.isLowercase = exports.isUppercase = exports.isEmail = exports.isFunction = exports.isDate = exports.isRegExp = exports.isArray = exports.isPlainObject = exports.isObject = exports.isString = exports.isInfinity = exports.isNotANumber = exports.isNumber = exports.isFalse = exports.isTrue = exports.isBoolean = exports.isNull = exports.isNotNull = undefined;
exports.createRegExpConstraint = createRegExpConstraint;
exports.createMinConstraint = createMinConstraint;
exports.createMaxConstraint = createMaxConstraint;
exports.createMinLengthConstraint = createMinLengthConstraint;
exports.createMaxLengthConstraint = createMaxLengthConstraint;

var _createConstraint = require('./util/createConstraint');

var _createConstraint2 = _interopRequireDefault(_createConstraint);

var _validators = require('./validators');

var validators = _interopRequireWildcard(_validators);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Type constraints

var isNotNull = exports.isNotNull = (0, _createConstraint2.default)(validators.isNotNull, 'Value is null or undefined');

var isNull = exports.isNull = (0, _createConstraint2.default)(validators.isNull, 'Value is neither null nor undefined');

var isBoolean = exports.isBoolean = (0, _createConstraint2.default)(validators.isBoolean, 'Value is not boolean');

var isTrue = exports.isTrue = (0, _createConstraint2.default)(validators.isTrue, 'Value is not true');

var isFalse = exports.isFalse = (0, _createConstraint2.default)(validators.isFalse, 'Value is not false');

var isNumber = exports.isNumber = (0, _createConstraint2.default)(validators.isNumber, 'Value is not number');

var isNotANumber = exports.isNotANumber = (0, _createConstraint2.default)(validators.isNotANumber, 'Value is not NaN');

var isInfinity = exports.isInfinity = (0, _createConstraint2.default)(validators.isInfinity, 'Value is not infinity');

var isString = exports.isString = (0, _createConstraint2.default)(validators.isString, 'Value is not string');

var isObject = exports.isObject = (0, _createConstraint2.default)(validators.isObject, 'Value is not object');

var isPlainObject = exports.isPlainObject = (0, _createConstraint2.default)(validators.isPlainObject, 'Value is not plain object');

var isArray = exports.isArray = (0, _createConstraint2.default)(validators.isArray, 'Value is not array');

var isRegExp = exports.isRegExp = (0, _createConstraint2.default)(validators.isRegExp, 'Value is not regular expression');

var isDate = exports.isDate = (0, _createConstraint2.default)(validators.isDate, 'Value is not date');

var isFunction = exports.isFunction = (0, _createConstraint2.default)(validators.isFunction, 'Value is not function');

// String constraints

function createRegExpConstraint(regexp) {
  var message = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return (0, _createConstraint2.default)(function (value) {
    return validators.isString(value) && regexp.test(value);
  }, message || 'Value is not matched by pattern');
}

var isEmail = exports.isEmail = (0, _createConstraint2.default)(validators.isEmail, 'Value is not valid email address');

var isUppercase = exports.isUppercase = (0, _createConstraint2.default)(validators.isUppercase, 'Value has symbols not in uppercase');

var isLowercase = exports.isLowercase = (0, _createConstraint2.default)(validators.isLowercase, 'Value has symbols not in lowercase');

// Number constraints

function createMinConstraint(min) {
  var message = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return (0, _createConstraint2.default)(function (value) {
    return value >= min;
  }, message || 'Value is less than ' + min);
}

function createMaxConstraint(max) {
  var message = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return (0, _createConstraint2.default)(function (value) {
    return value <= max;
  }, message || 'Value is more than ' + max);
}

// Length constraints

var isNotEmpty = exports.isNotEmpty = (0, _createConstraint2.default)(validators.isNotEmpty, 'Value is empty');

var isEmpty = exports.isEmpty = (0, _createConstraint2.default)(validators.isEmpty, 'Value is not empty');

function createMinLengthConstraint(minLength) {
  var message = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return (0, _createConstraint2.default)(function (value) {
    return value && value.length >= minLength;
  }, message || 'Value length is less than ' + minLength);
}

function createMaxLengthConstraint(maxLength) {
  var message = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return (0, _createConstraint2.default)(function (value) {
    return value && value.length <= maxLength;
  }, message || 'Value length is more than ' + maxLength);
}