'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _combineConstraints = require('../util/combineConstraints');

var _combineConstraints2 = _interopRequireDefault(_combineConstraints);

var _nullableConstraint = require('../util/nullableConstraint');

var _nullableConstraint2 = _interopRequireDefault(_nullableConstraint);

var _createArrayConstraint = require('../util/createArrayConstraint');

var _createArrayConstraint2 = _interopRequireDefault(_createArrayConstraint);

var _createObjectConstraint = require('../util/createObjectConstraint');

var _createObjectConstraint2 = _interopRequireDefault(_createObjectConstraint);

var _createConstraint = require('../util/createConstraint');

var _createConstraint2 = _interopRequireDefault(_createConstraint);

var _Schema = require('./Schema');

var _Schema2 = _interopRequireDefault(_Schema);

var _validators = require('../validators');

var validators = _interopRequireWildcard(_validators);

var _constraints = require('../constraints');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function field(name, constraints) {
  return new _Schema2.default({
    type: name,
    check: (0, _combineConstraints2.default)(constraints),
    children: null
  });
}

function ensureFieldIsSchema(maybeSchema) {
  var name = arguments.length <= 1 || arguments[1] === undefined ? 'field' : arguments[1];

  return maybeSchema instanceof _Schema2.default ? maybeSchema : field(name, maybeSchema);
}

function fields(fieldMap) {
  return Object.keys(fieldMap).reduce(function (acc, key) {
    return acc[key] = ensureFieldIsSchema(fieldMap[key], key), acc;
  }, {});
}

function object(fieldMap, name) {
  var children = fields(fieldMap);

  var schemaConstraint = (0, _createObjectConstraint2.default)(Object.keys(children).reduce(function (acc, key) {
    return acc[key] = children[key].check, acc;
  }, {}));

  return new _Schema2.default({
    type: name || 'object',
    check: schemaConstraint,
    children: children
  });
}

function optional(maybeSchema) {
  var schema = ensureFieldIsSchema(maybeSchema);

  return new _Schema2.default({
    type: 'optional(' + schema.type + ')',
    check: (0, _nullableConstraint2.default)(schema.check),
    children: schema
  });
}

function list(maybeSchema) {
  var schema = ensureFieldIsSchema(maybeSchema);

  return new _Schema2.default({
    type: 'list(' + schema.type + ')',
    check: (0, _createArrayConstraint2.default)(schema.check),
    children: schema
  });
}

function contextRoot(schema) {
  return new _Schema2.default({
    type: schema.type,
    check: function check(value, context) {
      return schema.check(value, { value: value, parent: context });
    },
    children: schema.children
  });
}

function type(name, fieldMap) {
  return contextRoot(object(fieldMap, name));
}

exports.default = {
  object: object,
  optional: optional,
  field: field,
  fields: fields,
  list: list,
  contextRoot: contextRoot,
  type: type,

  validators: validators,

  constraint: _createConstraint2.default,

  isNotNull: _constraints.isNotNull,
  isNull: _constraints.isNull,
  isBoolean: _constraints.isBoolean,
  isTrue: _constraints.isTrue,
  isFalse: _constraints.isFalse,
  isNumber: _constraints.isNumber,
  isNotANumber: _constraints.isNotANumber,
  isInfinity: _constraints.isInfinity,
  isString: _constraints.isString,
  isObject: _constraints.isObject,
  isPlainObject: _constraints.isPlainObject,
  isArray: _constraints.isArray,
  isRegExp: _constraints.isRegExp,
  isDate: _constraints.isDate,
  isFunction: _constraints.isFunction,
  regexp: _constraints.createRegExpConstraint,
  isEmail: _constraints.isEmail,
  isUppercase: _constraints.isUppercase,
  isLowercase: _constraints.isLowercase,
  min: _constraints.createMinConstraint,
  max: _constraints.createMaxConstraint,
  isNotEmpty: _constraints.isNotEmpty,
  isEmpty: _constraints.isEmpty,
  minLength: _constraints.createMinLengthConstraint,
  maxLength: _constraints.createMaxLengthConstraint
};