'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createArrayConstraint;

var _executeMessage = require('./executeMessage');

var _executeMessage2 = _interopRequireDefault(_executeMessage);

var _validators = require('../validators');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createArrayConstraint(constraint) {
  var message = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (value, context) {
    if (!(0, _validators.isArray)(value)) {
      return Promise.resolve({
        valid: false,
        message: (0, _executeMessage2.default)(value, context, [], message || 'Array is not valid'),
        children: []
      });
    }

    return Promise.all(value.map(function (valueItem) {
      return constraint(valueItem, context);
    })).then(function (children) {
      var valid = children.every(function (child) {
        return child.valid;
      });

      return {
        valid: valid,
        message: valid ? null : (0, _executeMessage2.default)(value, context, children, message || 'Array is not valid'),
        children: children
      };
    });
  };
}